from socket import socket, create_connection, AF_INET, SOCK_DGRAM
from struct import unpack
from time import time
from dataclasses import dataclass
from typing import Tuple

# * Dataclass
@dataclass
class Status:
    name: str
    map: str
    players: int
    wave: int
    version: float
    vertype: str
    ping: float

# * Главный класс
class Server:
    def __init__(
        self,
        server_host: str,
        server_port: int=6567,
        input_port: int=6859
    ) -> None:
        self.server: Tuple[str, int] = (server_host, server_port)
        self.input_server: Tuple[str, int] = (server_host, input_port)

    def get_status(self, timeout: float=10.0) -> Status:
        # * Инициализация сервера
        s = socket(AF_INET, SOCK_DGRAM)
        s.connect(self.server)
        s.settimeout(timeout)

        # * Создание и так понятно чего для чего
        info = {}

        # * Получение данных и замер
        s_time = time()
        s.send(b"\xfe\x01")
        data = s.recv(1024)
        e_time = time()

        # * Парсинг
        info["name"] = data[1:data[0]+1].decode("utf-8")
        data = data[data[0]+1:]
        info["map"] = data[1:data[0]+1].decode("utf-8")
        data = data[data[0]+1:]
        info["players"] = unpack(">i", data[:4])[0]
        data = data[4:]
        info["wave"] = unpack(">i", data[:4])[0]
        data = data[4:]
        info["version"] = unpack(">i", data[:4])[0]
        data = data[4:]
        info["vertype"] = data[1:data[0]+1].decode("utf-8")
        info["ping"] = round((e_time - s_time) * 1000)

        return Status(**info)
    
    def send_command(self, command: str) -> None:
        s = create_connection(self.input_server)
        s.sendall(
            bytes(
                command.encode(errors="ignore")
            )
        )
        s.close()
    
    def ping(self, timeout: float=10.0) -> int:
        # * Инициализация сервера
        s = socket(AF_INET, SOCK_DGRAM)
        s.connect(self.server)
        s.settimeout(timeout)

        # * Получение данных и замер
        s_time = time()
        s.send(b"\xfe\x01")
        s.recv(1024)
        e_time = time()

        return round((e_time - s_time) * 1000)



import os
import setuptools
import pydustry

# Функция получения полных путей к папкам и подпапкам
def globalizer(dirpath: str) -> list:
    files = []
    folder_abspath = os.path.abspath(dirpath)
    if os.path.isdir(folder_abspath):
        for i in os.listdir(folder_abspath):
            path = folder_abspath + os.sep + i
            if os.path.isdir(path):
                for _i in globalizer(path):
                    files.append(_i)
            elif os.path.isfile(path):
                files.append(path)
    elif os.path.isfile(folder_abspath):
        files.append(folder_abspath)
    return files

setuptools.setup(
	name=pydustry.__name__,
	version=pydustry.__version__,
	description='Module for server information in the game Mindustry.',
	keywords=pydustry.__keywords__,
	packages=setuptools.find_packages(),
	author_email='semina054@gmail.com',
	url="https://github.com/romanin-rf",
	long_description=open(os.path.join(os.path.dirname(__file__), 'README.md')).read(),
	long_description_content_type="text/markdown",
	author='ProgrammerFromParlament',
	license='MIT'
)